package com.qipeng.checkstyle.checks;

import com.puppycrawl.tools.checkstyle.api.AbstractCheck;
import com.puppycrawl.tools.checkstyle.api.DetailAST;
import com.puppycrawl.tools.checkstyle.api.TokenTypes;
import com.puppycrawl.tools.checkstyle.utils.CommonUtils;

/**
 * 单行注释内容空格检查类。
 *
 * @author tieguanyin
 */
@SuppressWarnings("all")
public class SpacedCommentCheck extends AbstractCheck {

  @Override
  public int[] getDefaultTokens() {
    return getAcceptableTokens();
  }

  @Override
  public int[] getAcceptableTokens() {
    return new int[]{TokenTypes.SINGLE_LINE_COMMENT};
  }

  @Override
  public int[] getRequiredTokens() {
    return CommonUtils.EMPTY_INT_ARRAY;
  }

  @Override
  public boolean isCommentNodesRequired() {
    return true;
  }

  @Override
  public void visitToken(DetailAST ast) {
    String comment = getCommentFirstLine(ast);
    if (!isBlank(comment) && !comment.startsWith(" ")) {
      log(ast.getLineNo(), "注释符 ''//'' 和注释内容之间应该保持一个空格。");
    }
  }

  private static String getCommentFirstLine(DetailAST ast) {
    String commentContent = "";
    DetailAST firstChild = ast.getFirstChild();
    if (firstChild != null && firstChild.getType() == TokenTypes.COMMENT_CONTENT) {
      commentContent = firstChild.getText();
    }
    return commentContent;
  }

  private static boolean isBlank(CharSequence cs) {
    int strLen = cs == null ? 0 : cs.length();
    if (strLen == 0) {
      return true;
    }

    for (int i = 0; i < strLen; i++) {
      if (!Character.isWhitespace(cs.charAt(i))) {
        return false;
      }
    }

    return true;
  }
}
